package codegen;
import parsing.Parser;
import parsing.nonterminals.NonTerminal;
import parsing.nonterminals.Label;

public class CodeGen
{

 public static CodeGen codegen;
 public static Parser parser;

 /**
 * constructor
 */
 public CodeGen(Parser parser)
 {
  codegen = this;
  CodeGen.parser = parser;
 }

 public static void createTemplate()
 {
  parser.println(".class public Compiled");
  parser.println(".super java/lang/Object");
  parser.println(".field public static compiled LCompiled");
  parser.println(".method public <init>()V");
  parser.emitLn(".limit stack 2");
  parser.emitLn(".limit locals 1");
  parser.emitLn("aload_0");
  parser.emitLn("invokespecial java/lang/Object/<init>()V");
  parser.emitLn("aload_0");
  parser.emitLn("putstatic Compiled/compiled LCompiled");
  parser.emitLn("return");
  parser.println(".end method");
 }

 public static void openMethod()
 {
  parser.println(".method public main()V");
  parser.emitLn(".limit stack 50");
  parser.emitLn(".limit locals 1");
 }

 public static void closeMethod()
 {
  parser.emitLn("return");
  parser.println(".end method");
 }

 public static void destroyApp()
 {
  parser.emitLn("getstatic Main/main LMain");
  parser.emitLn("iconst_1");
  parser.emitLn("invokevirtual Main/destroyApp(Z)V");
 }

 public static void alloc(String name)
 {
  if(parser.getLook()!='=')
  {
   CodeGen.addField(name, "I", "0");
  }
  else
  {
   String value = new String();
   parser.match('=');
   if(parser.getLook()=='-')
   {
    parser.match('-');
    parser.getNum();
    value = '-'+parser.getValue();
   }
   else
   {
    parser.getNum();
    value = parser.getValue();
   }
   CodeGen.addField(name, "I", value);
  }
 }

 public static void addField(String fieldName, String fieldType, String value)
 {
  final String s = new String(".class public Compiled\n.super java/lang/Object\n");
  final StringBuffer sb = parser.getResult();
  final String str = ".field public "+fieldName+" "+fieldType+" = "+value+'\n';
  sb.insert(s.length(), str);
  parser.setResult(sb);
 }

 public static void readVar()
 {
  parser.emitLn("invokestatic RTL/read()V");
  store(parser.getValue());
 }

 public static void write()
 {
  loadVar(parser.getValue());
  parser.emitLn("invokestatic RTL/write(I)V");
 }

 public static void clear()
 {
  parser.emitLn("iconst_0");
 }

 public static void negate()
 {
  parser.emitLn("iload_1");
 }

 public static void loadConst(String s)
 {
  parser.emitLn("ldc "+s);
 }

 public static void loadVar(String name)
 {
  if(!NonTerminal.inTable(name)) parser.abort("Variable "+name+" is undefined");
  else
  {
   parser.emitLn("aload_0");
   parser.emitLn("getfield Compiled/"+name+" I");
  }
 }

 public static void popAdd()
 {
  parser.emitLn("iadd");
 }

 public static void popSub()
 {
  parser.emitLn("swap");
  parser.emitLn("isub");
 }

 public static void popMul()
 {
  parser.emitLn("imul");
 }

 public static void popDiv()
 {
  parser.emitLn("swap");
  parser.emitLn("idiv");
 }

 public static void store(String name)
 {
  if(!NonTerminal.inTable(name)) parser.abort("Variable "+name+" is undefined");
  else
  {
   parser.emitLn("aload_0");
   parser.emitLn("putfield Compiled/"+name+" I");
  }
 }

 public static void notIt()
 {
  parser.emitLn("iconst_1");
  parser.emitLn("ixor");
 }

 public static void popAnd()
 {
  parser.emitLn("iand");
 }

 public static void popOr()
 {
  parser.emitLn("ior");
 }

 public static void popXor()
 {
  parser.emitLn("ixor");
 }

 public static void setEqual()
 {
  Label label1 = new Label(CodeGen.parser);
  Label label2 = new Label(CodeGen.parser);
  parser.emitLn("if_icmpne "+label1.getName());
  loadConst("-1");
  parser.emitLn("goto "+label2.getName());
  label1.printLabel();
  clear();
  parser.emitLn("goto "+label2.getName());
  label2.printLabel();
 }

 public static void setNotEqual()
 {
  Label label1 = new Label(CodeGen.parser);
  Label label2 = new Label(CodeGen.parser);
  parser.emitLn("if_icmpeq "+label1.getName());
  loadConst("-1");
  parser.emitLn("goto "+label2.getName());
  label1.printLabel();
  clear();
  parser.emitLn("goto "+label2.getName());
  label2.printLabel();
 }

 public static void setLess()
 {
  Label label1 = new Label(CodeGen.parser);
  Label label2 = new Label(CodeGen.parser);
  parser.emitLn("if_icmpge "+label1.getName());
  loadConst("-1");
  parser.emitLn("goto "+label2.getName());
  label1.printLabel();
  clear();
  parser.emitLn("goto "+label2.getName());
  label2.printLabel();
 }

 public static void setGreater()
 {
  Label label1 = new Label(CodeGen.parser);
  Label label2 = new Label(CodeGen.parser);
  parser.emitLn("if_icmple "+label1.getName());
  loadConst("-1");
  parser.emitLn("goto "+label2.getName());
  label1.printLabel();
  clear();
  parser.emitLn("goto "+label2.getName());
  label2.printLabel();
 }

 public static void setLessOrEqual()
 {
  Label label1 = new Label(CodeGen.parser);
  Label label2 = new Label(CodeGen.parser);
  parser.emitLn("if_icmple "+label1.getName());
  clear();
  parser.emitLn("goto "+label2.getName());
  label1.printLabel();
  loadConst("-1");
  parser.emitLn("goto "+label2.getName());
  label2.printLabel();
 }

 public static void setGreaterOrEquals()
 {
  Label label1 = new Label(CodeGen.parser);
  Label label2 = new Label(CodeGen.parser);
  parser.emitLn("if_icmpge "+label1.getName());
  clear();
  parser.emitLn("goto "+label2.getName());
  label1.printLabel();
  loadConst("-1");
  parser.emitLn("goto "+label2.getName());
  label2.printLabel();
 }

 public static void branch(String labelname)
 {
  parser.emitLn("goto "+labelname);
 }

 public static void branchFalse(String labelname)
 {
  parser.emitLn("ifeq "+labelname);
 }
}




